/**
 * DevExtreme (cjs/renovation/ui/scroll_view/scrollable_utils.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
exports.ensureLocation = ensureLocation;
exports.getMaxScrollOffset = getMaxScrollOffset;
exports.getBoundaryProps = getBoundaryProps;
exports.normalizeCoordinate = normalizeCoordinate;
exports.getPublicCoordinate = getPublicCoordinate;
exports.normalizeLocation = normalizeLocation;
exports.getElementLocation = getElementLocation;
exports.updateAllowedDirection = updateAllowedDirection;
var _type = require("../../../core/utils/type");
var _scroll_rtl_behavior = _interopRequireDefault(require("../../../core/utils/scroll_rtl_behavior"));
var _inflector = require("../../../core/utils/inflector");
var _common = require("../../../core/utils/common");
var _consts = require("./common/consts");
var _scroll_direction = require("./utils/scroll_direction");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _extends() {
    _extends = Object.assign || function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}

function ensureLocation(location) {
    if ((0, _type.isNumeric)(location)) {
        return {
            left: location,
            top: location
        }
    }
    return _extends({
        top: 0,
        left: 0
    }, location)
}

function getRelativeLocation(element) {
    var result = {
        top: 0,
        left: 0
    };
    var targetElement = element;
    while (!targetElement.matches(".".concat(_consts.SCROLLABLE_CONTENT_CLASS))) {
        result.top += targetElement.offsetTop;
        result.left += targetElement.offsetLeft;
        targetElement = targetElement.offsetParent
    }
    return result
}

function getMaxScrollOffset(dimension, containerRef) {
    return containerRef["scroll".concat((0, _inflector.titleize)(dimension))] - containerRef["client".concat((0, _inflector.titleize)(dimension))]
}

function getBoundaryProps(direction, scrollOffset, element, topPocketHeight) {
    var left = scrollOffset.left,
        top = scrollOffset.top;
    var boundaryProps = {};
    var _ScrollDirection = new _scroll_direction.ScrollDirection(direction),
        isHorizontal = _ScrollDirection.isHorizontal,
        isVertical = _ScrollDirection.isVertical;
    if (isHorizontal) {
        boundaryProps.reachedLeft = left <= 0;
        boundaryProps.reachedRight = Math.round(left) >= getMaxScrollOffset("width", element)
    }
    if (isVertical) {
        boundaryProps.reachedTop = top <= 0;
        boundaryProps.reachedBottom = getMaxScrollOffset("height", element) - topPocketHeight - top < .5
    }
    return boundaryProps
}

function getScrollBarSize(dimension, containerRef) {
    return containerRef["offset".concat(dimension)] - containerRef["client".concat(dimension)]
}

function needNormalizeCoordinate(prop, rtlEnabled) {
    return true === rtlEnabled && "left" === prop
}

function normalizeCoordinate(prop, coordinate, rtlEnabled) {
    return needNormalizeCoordinate(prop, rtlEnabled) && (0, _scroll_rtl_behavior.default)().positive ? -1 * coordinate : coordinate
}

function getPublicCoordinate(prop, coordinate, maxLeftOffset, rtlEnabled) {
    return needNormalizeCoordinate(prop, rtlEnabled) ? maxLeftOffset + normalizeCoordinate(prop, coordinate, rtlEnabled) : coordinate
}

function normalizeLocation(location, direction) {
    if ((0, _type.isPlainObject)(location)) {
        return {
            left: -(0, _common.ensureDefined)(location.left, location.x),
            top: -(0, _common.ensureDefined)(location.top, location.y)
        }
    }
    var _ScrollDirection2 = new _scroll_direction.ScrollDirection(direction),
        isHorizontal = _ScrollDirection2.isHorizontal,
        isVertical = _ScrollDirection2.isVertical;
    return {
        left: isHorizontal ? -location : void 0,
        top: isVertical ? -location : void 0
    }
}

function getElementLocationInternal(element, prop, offset, direction, containerRef, rtlEnabled) {
    var dimension = direction === _consts.DIRECTION_VERTICAL ? "Height" : "Width";
    var relativeLocation = getRelativeLocation(element)[prop];
    var scrollBarSize = getScrollBarSize(dimension, containerRef);
    var containerSize = containerRef["offset".concat(dimension)];
    var elementOffset = element["offset".concat(dimension)];
    var offsetStart = offset[prop];
    var offsetEnd = offset[direction === _consts.DIRECTION_VERTICAL ? "bottom" : "right"] || 0;
    var containerLocation = normalizeCoordinate(prop, containerRef["scroll".concat((0, _inflector.titleize)(prop))], rtlEnabled);
    if (relativeLocation < containerLocation + offsetStart) {
        if (elementOffset < containerSize - offsetStart - offsetEnd) {
            return relativeLocation - offsetStart
        }
        return relativeLocation + elementOffset - containerSize + offsetEnd + scrollBarSize
    }
    if (relativeLocation + elementOffset >= containerLocation + containerSize - offsetEnd - scrollBarSize) {
        if (elementOffset < containerSize - offsetStart - offsetEnd) {
            return relativeLocation + elementOffset + scrollBarSize - containerSize + offsetEnd
        }
        return relativeLocation - offsetStart
    }
    return containerLocation
}

function getElementLocation(element, offset, direction, containerRef, rtlEnabled) {
    var prop = direction === _consts.DIRECTION_VERTICAL ? "top" : "left";
    var maxScrollLeftOffset = getMaxScrollOffset("width", containerRef);
    var location = normalizeCoordinate(prop, getElementLocationInternal(element, prop, offset, direction, containerRef, rtlEnabled), rtlEnabled);
    return getPublicCoordinate(prop, location, maxScrollLeftOffset, rtlEnabled)
}

function updateAllowedDirection(allowedDirections, direction) {
    var _ScrollDirection3 = new _scroll_direction.ScrollDirection(direction),
        isBoth = _ScrollDirection3.isBoth,
        isHorizontal = _ScrollDirection3.isHorizontal,
        isVertical = _ScrollDirection3.isVertical;
    if (isBoth && allowedDirections.vertical && allowedDirections.horizontal) {
        return _consts.DIRECTION_BOTH
    }
    if (isHorizontal && allowedDirections.horizontal) {
        return _consts.DIRECTION_HORIZONTAL
    }
    if (isVertical && allowedDirections.vertical) {
        return _consts.DIRECTION_VERTICAL
    }
    return
}
