/**
 * DevExtreme (esm/renovation/ui/scroll_view/scrollable_utils.js)
 * Version: 21.1.2
 * Build date: Fri Apr 09 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import {
    isNumeric,
    isPlainObject
} from "../../../core/utils/type";
import getScrollRtlBehavior from "../../../core/utils/scroll_rtl_behavior";
import {
    titleize
} from "../../../core/utils/inflector";
import {
    ensureDefined
} from "../../../core/utils/common";
import {
    SCROLLABLE_CONTENT_CLASS,
    DIRECTION_VERTICAL,
    DIRECTION_HORIZONTAL,
    DIRECTION_BOTH
} from "./common/consts";
import {
    ScrollDirection
} from "./utils/scroll_direction";
export function ensureLocation(location) {
    if (isNumeric(location)) {
        return {
            left: location,
            top: location
        }
    }
    return _extends({
        top: 0,
        left: 0
    }, location)
}

function getRelativeLocation(element) {
    var result = {
        top: 0,
        left: 0
    };
    var targetElement = element;
    while (!targetElement.matches(".".concat(SCROLLABLE_CONTENT_CLASS))) {
        result.top += targetElement.offsetTop;
        result.left += targetElement.offsetLeft;
        targetElement = targetElement.offsetParent
    }
    return result
}
export function getMaxScrollOffset(dimension, containerRef) {
    return containerRef["scroll".concat(titleize(dimension))] - containerRef["client".concat(titleize(dimension))]
}
export function getBoundaryProps(direction, scrollOffset, element, topPocketHeight) {
    var {
        left: left,
        top: top
    } = scrollOffset;
    var boundaryProps = {};
    var {
        isHorizontal: isHorizontal,
        isVertical: isVertical
    } = new ScrollDirection(direction);
    if (isHorizontal) {
        boundaryProps.reachedLeft = left <= 0;
        boundaryProps.reachedRight = Math.round(left) >= getMaxScrollOffset("width", element)
    }
    if (isVertical) {
        boundaryProps.reachedTop = top <= 0;
        boundaryProps.reachedBottom = getMaxScrollOffset("height", element) - topPocketHeight - top < .5
    }
    return boundaryProps
}

function getScrollBarSize(dimension, containerRef) {
    return containerRef["offset".concat(dimension)] - containerRef["client".concat(dimension)]
}

function needNormalizeCoordinate(prop, rtlEnabled) {
    return true === rtlEnabled && "left" === prop
}
export function normalizeCoordinate(prop, coordinate, rtlEnabled) {
    return needNormalizeCoordinate(prop, rtlEnabled) && getScrollRtlBehavior().positive ? -1 * coordinate : coordinate
}
export function getPublicCoordinate(prop, coordinate, maxLeftOffset, rtlEnabled) {
    return needNormalizeCoordinate(prop, rtlEnabled) ? maxLeftOffset + normalizeCoordinate(prop, coordinate, rtlEnabled) : coordinate
}
export function normalizeLocation(location, direction) {
    if (isPlainObject(location)) {
        return {
            left: -ensureDefined(location.left, location.x),
            top: -ensureDefined(location.top, location.y)
        }
    }
    var {
        isHorizontal: isHorizontal,
        isVertical: isVertical
    } = new ScrollDirection(direction);
    return {
        left: isHorizontal ? -location : void 0,
        top: isVertical ? -location : void 0
    }
}

function getElementLocationInternal(element, prop, offset, direction, containerRef, rtlEnabled) {
    var dimension = direction === DIRECTION_VERTICAL ? "Height" : "Width";
    var relativeLocation = getRelativeLocation(element)[prop];
    var scrollBarSize = getScrollBarSize(dimension, containerRef);
    var containerSize = containerRef["offset".concat(dimension)];
    var elementOffset = element["offset".concat(dimension)];
    var offsetStart = offset[prop];
    var offsetEnd = offset[direction === DIRECTION_VERTICAL ? "bottom" : "right"] || 0;
    var containerLocation = normalizeCoordinate(prop, containerRef["scroll".concat(titleize(prop))], rtlEnabled);
    if (relativeLocation < containerLocation + offsetStart) {
        if (elementOffset < containerSize - offsetStart - offsetEnd) {
            return relativeLocation - offsetStart
        }
        return relativeLocation + elementOffset - containerSize + offsetEnd + scrollBarSize
    }
    if (relativeLocation + elementOffset >= containerLocation + containerSize - offsetEnd - scrollBarSize) {
        if (elementOffset < containerSize - offsetStart - offsetEnd) {
            return relativeLocation + elementOffset + scrollBarSize - containerSize + offsetEnd
        }
        return relativeLocation - offsetStart
    }
    return containerLocation
}
export function getElementLocation(element, offset, direction, containerRef, rtlEnabled) {
    var prop = direction === DIRECTION_VERTICAL ? "top" : "left";
    var maxScrollLeftOffset = getMaxScrollOffset("width", containerRef);
    var location = normalizeCoordinate(prop, getElementLocationInternal(element, prop, offset, direction, containerRef, rtlEnabled), rtlEnabled);
    return getPublicCoordinate(prop, location, maxScrollLeftOffset, rtlEnabled)
}
export function updateAllowedDirection(allowedDirections, direction) {
    var {
        isBoth: isBoth,
        isHorizontal: isHorizontal,
        isVertical: isVertical
    } = new ScrollDirection(direction);
    if (isBoth && allowedDirections.vertical && allowedDirections.horizontal) {
        return DIRECTION_BOTH
    }
    if (isHorizontal && allowedDirections.horizontal) {
        return DIRECTION_HORIZONTAL
    }
    if (isVertical && allowedDirections.vertical) {
        return DIRECTION_VERTICAL
    }
    return
}
